@@ -3,6 +3,7 @@
 mod agent;
 #[cfg(any(feature = "collector_client", feature = "wasm_collector_client"))]
 mod collector;
+pub(crate) mod runtime;
 #[allow(clippy::all, unreachable_pub, dead_code)]
 #[rustfmt::skip]
 mod thrift;
@@ -10,11 +11,13 @@ mod env;
 pub(crate) mod transport;
 mod uploader;
 
+use self::runtime::JaegerTraceRuntime;
 use self::thrift::jaeger;
 use agent::AgentAsyncClientUdp;
 use async_trait::async_trait;
 #[cfg(any(feature = "collector_client", feature = "wasm_collector_client"))]
 use collector::CollectorAsyncClientHttp;
+use opentelemetry_semantic_conventions as semcov;
 
 #[cfg(feature = "isahc_collector_client")]
 #[allow(unused_imports)] // this is actually used to configure authentication
@@ -25,7 +28,6 @@ use opentelemetry::trace::TraceError;
 use opentelemetry::{
     global, sdk,
     sdk::export::trace,
-    sdk::trace::TraceRuntime,
     trace::{Event, Link, SpanKind, StatusCode, TracerProvider},
     Key, KeyValue,
 };
@@ -35,7 +37,7 @@ use std::{
     net,
     time::{Duration, SystemTime},
 };
-use uploader::BatchUploader;
+use uploader::{AsyncUploader, SyncUploader, Uploader};
 
 #[cfg(all(
     any(
@@ -46,9 +48,11 @@ use uploader::BatchUploader;
     not(feature = "isahc_collector_client")
 ))]
 use headers::authorization::Credentials;
-
-/// Default service name if no service is configured.
-const DEFAULT_SERVICE_NAME: &str = "OpenTelemetry";
+use opentelemetry::sdk::resource::ResourceDetector;
+use opentelemetry::sdk::resource::SdkProvidedResourceDetector;
+use opentelemetry::sdk::trace::Config;
+use opentelemetry::sdk::Resource;
+use std::sync::Arc;
 
 /// Default agent endpoint if none is provided
 const DEFAULT_AGENT_ENDPOINT: &str = "127.0.0.1:6831";
@@ -70,7 +74,7 @@ pub struct Exporter {
     process: jaeger::Process,
     /// Whether or not to export instrumentation information.
     export_instrumentation_lib: bool,
-    uploader: uploader::BatchUploader,
+    uploader: Box<dyn Uploader>,
 }
 
 /// Jaeger process configuration
@@ -124,7 +128,8 @@ pub struct PipelineBuilder {
     #[cfg(feature = "collector_client")]
     client: Option<Box<dyn HttpClient>>,
     export_instrument_library: bool,
-    process: Process,
+    service_name: Option<String>,
+    tags: Option<Vec<KeyValue>>,
     max_packet_size: Option<usize>,
     config: Option<sdk::trace::Config>,
 }
@@ -143,10 +148,8 @@ impl Default for PipelineBuilder {
             #[cfg(feature = "collector_client")]
             client: None,
             export_instrument_library: true,
-            process: Process {
-                service_name: DEFAULT_SERVICE_NAME.to_string(),
-                tags: Vec::new(),
-            },
+            service_name: None,
+            tags: None,
             max_packet_size: None,
             config: None,
         };
@@ -226,13 +229,13 @@ impl PipelineBuilder {
 
     /// Assign the process service name.
     pub fn with_service_name<T: Into<String>>(mut self, service_name: T) -> Self {
-        self.process.service_name = service_name.into();
+        self.service_name = Some(service_name.into());
         self
     }
 
     /// Assign the process service tags.
     pub fn with_tags<T: IntoIterator<Item = KeyValue>>(mut self, tags: T) -> Self {
-        self.process.tags = tags.into_iter().collect();
+        self.tags = Some(tags.into_iter().collect());
         self
     }
 
@@ -261,58 +264,108 @@ impl PipelineBuilder {
     pub fn install_simple(self) -> Result<sdk::trace::Tracer, TraceError> {
         let tracer_provider = self.build_simple()?;
         let tracer =
-            tracer_provider.get_tracer("opentelemetry-jaeger", Some(env!("CARGO_PKG_VERSION")));
+            tracer_provider.tracer("opentelemetry-jaeger", Some(env!("CARGO_PKG_VERSION")));
         let _ = global::set_tracer_provider(tracer_provider);
         Ok(tracer)
     }
 
     /// Install a Jaeger pipeline with a batch span processor using the specified runtime.
-    pub fn install_batch<R: TraceRuntime>(
+    pub fn install_batch<R: JaegerTraceRuntime>(
         self,
         runtime: R,
     ) -> Result<sdk::trace::Tracer, TraceError> {
         let tracer_provider = self.build_batch(runtime)?;
         let tracer =
-            tracer_provider.get_tracer("opentelemetry-jaeger", Some(env!("CARGO_PKG_VERSION")));
+            tracer_provider.tracer("opentelemetry-jaeger", Some(env!("CARGO_PKG_VERSION")));
         let _ = global::set_tracer_provider(tracer_provider);
         Ok(tracer)
     }
 
+    // To reduce the overhead of copying service name in every spans. We remove service.name
+    // from the resource in config. Instead, we store it in process.
+    // The service name tag will attch to spans when it's exported.
+    fn build_config_and_process(&mut self) -> (Config, Process) {
+        let service_name = self.service_name.take();
+        if let Some(service_name) = service_name {
+            let config = if let Some(mut cfg) = self.config.take() {
+                cfg.resource = cfg.resource.map(|r| {
+                    let without_service_name = r
+                        .iter()
+                        .filter(|(k, _v)| **k != semcov::resource::SERVICE_NAME)
+                        .map(|(k, v)| KeyValue::new(k.clone(), v.clone()))
+                        .collect::<Vec<KeyValue>>();
+                    Arc::new(Resource::new(without_service_name))
+                });
+                cfg
+            } else {
+                Config {
+                    resource: Some(Arc::new(Resource::empty())),
+                    ..Default::default()
+                }
+            };
+            (
+                config,
+                Process {
+                    service_name,
+                    tags: self.tags.take().unwrap_or_default(),
+                },
+            )
+        } else {
+            let service_name = SdkProvidedResourceDetector
+                .detect(Duration::from_secs(0))
+                .get(semcov::resource::SERVICE_NAME)
+                .unwrap()
+                .to_string();
+            (
+                Config {
+                    // use a empty resource to prevent TracerProvider to assign a service name.
+                    resource: Some(Arc::new(Resource::empty())),
+                    ..Default::default()
+                },
+                Process {
+                    service_name,
+                    tags: self.tags.take().unwrap_or_default(),
+                },
+            )
+        }
+    }
+
     /// Build a configured `sdk::trace::TracerProvider` with a simple span processor.
     pub fn build_simple(mut self) -> Result<sdk::trace::TracerProvider, TraceError> {
-        let config = self.config.take();
-        let exporter = self.init_exporter()?;
+        let (config, process) = self.build_config_and_process();
+        let exporter = self.init_sync_exporter_with_process(process)?;
         let mut builder = sdk::trace::TracerProvider::builder().with_simple_exporter(exporter);
-        if let Some(config) = config {
-            builder = builder.with_config(config)
-        }
+        builder = builder.with_config(config);
 
         Ok(builder.build())
     }
 
     /// Build a configured `sdk::trace::TracerProvider` with a batch span processor using the
     /// specified runtime.
-    pub fn build_batch<R: TraceRuntime>(
+    pub fn build_batch<R: JaegerTraceRuntime>(
         mut self,
         runtime: R,
     ) -> Result<sdk::trace::TracerProvider, TraceError> {
-        let config = self.config.take();
-        let exporter = self.init_exporter()?;
+        let (config, process) = self.build_config_and_process();
+        let exporter = self.init_async_exporter_with_process(process, runtime.clone())?;
         let mut builder =
             sdk::trace::TracerProvider::builder().with_batch_exporter(exporter, runtime);
-        if let Some(config) = config {
-            builder = builder.with_config(config)
-        }
+        builder = builder.with_config(config);
 
         Ok(builder.build())
     }
 
-    /// Initialize a new exporter.
+    /// Initialize a new simple exporter.
     ///
     /// This is useful if you are manually constructing a pipeline.
-    pub fn init_exporter(self) -> Result<Exporter, TraceError> {
+    pub fn init_sync_exporter(mut self) -> Result<Exporter, TraceError> {
+        let (_, process) = self.build_config_and_process();
+        self.init_sync_exporter_with_process(process)
+    }
+
+    fn init_sync_exporter_with_process(self, process: Process) -> Result<Exporter, TraceError> {
         let export_instrumentation_lib = self.export_instrument_library;
-        let (process, uploader) = self.init_uploader()?;
+        let uploader = self.init_sync_uploader()?;
 
         Ok(Exporter {
             process: process.into(),
@@ -321,15 +374,58 @@ impl PipelineBuilder {
         })
     }
 
+    /// Initialize a new exporter.
+    ///
+    /// This is useful if you are manually constructing a pipeline.
+    pub fn init_async_exporter<R: JaegerTraceRuntime>(
+        mut self,
+        runtime: R,
+    ) -> Result<Exporter, TraceError> {
+        let (_, process) = self.build_config_and_process();
+        self.init_async_exporter_with_process(process, runtime)
+    }
+
+    fn init_async_exporter_with_process<R: JaegerTraceRuntime>(
+        self,
+        process: Process,
+        runtime: R,
+    ) -> Result<Exporter, TraceError> {
+        let export_instrumentation_lib = self.export_instrument_library;
+        let uploader = self.init_async_uploader(runtime)?;
+
+        Ok(Exporter {
+            process: process.into(),
+            export_instrumentation_lib,
+            uploader,
+        })
+    }
+
+    fn init_sync_uploader(self) -> Result<Box<dyn Uploader>, TraceError> {
+        let agent =
+            agent::AgentSyncClientUdp::new(self.agent_endpoint.as_slice(), self.max_packet_size)
+                .map_err::<Error, _>(Into::into)?;
+        Ok(Box::new(SyncUploader::Agent(agent)))
+    }
+
     #[cfg(not(any(feature = "collector_client", feature = "wasm_collector_client")))]
-    fn init_uploader(self) -> Result<(Process, BatchUploader), TraceError> {
-        let agent = AgentAsyncClientUdp::new(self.agent_endpoint.as_slice(), self.max_packet_size)
-            .map_err::<Error, _>(Into::into)?;
-        Ok((self.process, BatchUploader::Agent(agent)))
+    fn init_async_uploader<R: JaegerTraceRuntime>(
+        self,
+        runtime: R,
+    ) -> Result<Box<dyn Uploader>, TraceError> {
+        let agent = AgentAsyncClientUdp::new(
+            self.agent_endpoint.as_slice(),
+            self.max_packet_size,
+            runtime,
+        )
+        .map_err::<Error, _>(Into::into)?;
+        Ok(Box::new(AsyncUploader::Agent(agent)))
     }
 
     #[cfg(feature = "collector_client")]
-    fn init_uploader(self) -> Result<(Process, uploader::BatchUploader), TraceError> {
+    fn init_async_uploader<R: JaegerTraceRuntime>(
+        self,
+        runtime: R,
+    ) -> Result<Box<dyn Uploader>, TraceError> {
         if let Some(collector_endpoint) = self
             .collector_endpoint
             .transpose()
@@ -412,17 +508,21 @@ impl PipelineBuilder {
             });
 
             let collector = CollectorAsyncClientHttp::new(collector_endpoint, client);
-            Ok((self.process, uploader::BatchUploader::Collector(collector)))
+            let uploader: AsyncUploader<R> = AsyncUploader::Collector(collector);
+            Ok(Box::new(uploader))
         } else {
             let endpoint = self.agent_endpoint.as_slice();
-            let agent = AgentAsyncClientUdp::new(endpoint, self.max_packet_size)
+            let agent = AgentAsyncClientUdp::new(endpoint, self.max_packet_size, runtime)
                 .map_err::<Error, _>(Into::into)?;
-            Ok((self.process, BatchUploader::Agent(agent)))
+            Ok(Box::new(AsyncUploader::Agent(agent)))
         }
     }
 
     #[cfg(all(feature = "wasm_collector_client", not(feature = "collector_client")))]
-    fn init_uploader(self) -> Result<(Process, uploader::BatchUploader), TraceError> {
+    fn init_async_uploader<R: JaegerTraceRuntime>(
+        self,
+        runtime: R,
+    ) -> Result<Box<dyn Uploader>, TraceError> {
         if let Some(collector_endpoint) = self
             .collector_endpoint
             .transpose()
@@ -434,12 +534,12 @@ impl PipelineBuilder {
                 self.collector_password,
             )
             .map_err::<Error, _>(Into::into)?;
-            Ok((self.process, uploader::BatchUploader::Collector(collector)))
+            Ok(Box::new(AsyncUploader::Collector(collector)))
         } else {
             let endpoint = self.agent_endpoint.as_slice();
             let agent = AgentAsyncClientUdp::new(endpoint, self.max_packet_size)
                 .map_err::<Error, _>(Into::into)?;
-            Ok((self.process, BatchUploader::Agent(agent)))
+            Ok(Box::new(AsyncUploader::Agent(agent)))
         }
     }
 }
@@ -664,10 +764,11 @@ impl ExportError for Error {
 }
 
 #[cfg(test)]
-#[cfg(feature = "collector_client")]
+#[cfg(all(feature = "collector_client", feature = "rt-tokio"))]
 mod collector_client_tests {
     use crate::exporter::thrift::jaeger::Batch;
     use crate::new_pipeline;
+    use opentelemetry::runtime::Tokio;
     use opentelemetry::trace::TraceError;
 
     mod test_http_client {
@@ -695,10 +796,11 @@ mod collector_client_tests {
 
     #[test]
     fn test_bring_your_own_client() -> Result<(), TraceError> {
-        let (process, mut uploader) = new_pipeline()
+        let mut builder = new_pipeline()
             .with_collector_endpoint("localhost:6831")
-            .with_http_client(test_http_client::TestHttpClient)
-            .init_uploader()?;
+            .with_http_client(test_http_client::TestHttpClient);
+        let (_, process) = builder.build_config_and_process();
+        let mut uploader = builder.init_async_uploader(Tokio)?;
         let res = futures::executor::block_on(async {
             uploader
                 .upload(Batch::new(process.into(), Vec::new()))
@@ -722,7 +824,7 @@ mod collector_client_tests {
     fn test_set_collector_endpoint() {
         let invalid_uri = new_pipeline()
             .with_collector_endpoint("127.0.0.1:14268/api/traces")
-            .init_uploader();
+            .init_async_uploader(Tokio);
         assert!(invalid_uri.is_err());
         assert_eq!(
             format!("{:?}", invalid_uri.err().unwrap()),
@@ -731,7 +833,7 @@ mod collector_client_tests {
 
         let valid_uri = new_pipeline()
             .with_collector_endpoint("http://127.0.0.1:14268/api/traces")
-            .init_uploader();
+            .init_async_uploader(Tokio);
 
         assert!(valid_uri.is_ok());
     }
