@@ -48,7 +48,7 @@
 //! ```toml
 //! [dependencies]
 //! opentelemetry = { version = "*", features = ["rt-tokio"] }
-//! opentelemetry-jaeger = { version = "*", features = ["tokio"] }
+//! opentelemetry-jaeger = { version = "*", features = ["rt-tokio"] }
 //! ```
 //!
 //! ```no_run
@@ -162,6 +162,17 @@
 //!
 //! * `wasm_collector_client`: Enable collector in wasm.
 //!
+//! Support for recording and exporting telemetry asynchronously can be added
+//! via the following flags, it extends the [`opentelemetry`] feature:
+//!
+//! * `rt-tokio`: Enable sending UDP packets to Jaeger agent asynchronously when [`Tokio`] runtime is used.
+//!
+//! * `rt-tokio-current-thread`: Enable sending UDP packets to Jaeger agent asynchronously when [`TokioCurrentThread`] runtime is used.
+//!
+//! * `rt-async-std`: Enable sending UDP packets to Jaeger agent asynchronously when [`AsyncStd`] runtime is used.
+//!
+//! [`opentelemetry`]: https://crates.io/crates/opentelemetry
+//!
 //! ## Supported Rust Versions
 //!
 //! OpenTelemetry is built against the latest stable release. The minimum
@@ -184,7 +195,7 @@
     unreachable_pub,
     unused
 )]
-#![cfg_attr(docsrs, feature(doc_cfg), deny(broken_intra_doc_links))]
+#![cfg_attr(docsrs, feature(doc_cfg), deny(rustdoc::broken_intra_doc_links))]
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/open-telemetry/opentelemetry-rust/main/assets/logo.svg"
 )]
@@ -581,5 +592,7 @@ mod propagator {
     }
 }
 
-pub use exporter::{new_pipeline, Error, Exporter, PipelineBuilder, Process};
+pub use exporter::{
+    new_pipeline, runtime::JaegerTraceRuntime, Error, Exporter, PipelineBuilder, Process,
+};
 pub use propagator::Propagator;
